# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

funC <- function(F) {
    .Call(`_VUROCS_funC`, F)
}

funC1 <- function(F) {
    .Call(`_VUROCS_funC1`, F)
}

funC2 <- function(F) {
    .Call(`_VUROCS_funC2`, F)
}

funD <- function(F) {
    .Call(`_VUROCS_funD`, F)
}

funD1 <- function(F) {
    .Call(`_VUROCS_funD1`, F)
}

funD2 <- function(F) {
    .Call(`_VUROCS_funD2`, F)
}

